/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  Copyright (c) 2011                                                     **
**  University of Oregon, Eugene, USA                                      **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



package scalasca.cubex.cube;
import java.lang.*;
import java.util.*;
import scalasca.cubex.cube.errors.*;
import scalasca.cubex.cube.datalayout.index.*;
import scalasca.cubex.cube.services.*;
import scalasca.cubex.cube.services.transformation.*;
import scalasca.cubex.cube.datalayout.*;
import scalasca.cubex.cube.datalayout.data.*;
import scalasca.cubex.cube.datalayout.data.value.*;


/*
 * Class , delivers file structure of CubeX
 *
 */
public class Metric extends NamedVertex implements ProgressReportInterface
{
    private ProgressReportContext progressContext = new ProgressReportContext();

    protected String display_name;
    ;
    protected String         uom;
    protected String         dtype;
    protected String         url;
    protected String         description;
    protected Value          metric_value;
    protected Vector<Double> metric_value_params;
    protected Matrix         matrix;

    private double progress;                    // from 0 to 1. status of "createDataMatrix"
    private String progress_message;

    protected CnodesEnumeration enumeration;


    //     @SuppressWarnings("unchecked")
    public Metric(
        int    _id,
        Metric _parent
        )
    {
        super( _id, "", _parent );
    }

    public void
    init(
        String _uniq_name,
        String _display_name,
        String _dtype,
        String _uom,
        String _url,
        String _description
        ) throws UnknownValueTypeException, WrongNumberOfValueParametersException
    {
        setName( _uniq_name );
        display_name        = _display_name;
        uom                 = _uom;
        dtype               = _dtype;
        url                 = _url;
        description         = _description;
        metric_value        = Value.getValue( ValueType.getValueType( dtype ) );
        metric_value_params = ValueType.getValueParameters( dtype );
        metric_value.setValueParameters( metric_value_params );
    }



    public double
    getProgress()
    {
        return progressContext.getProgress();
    }

    public String
    getProgressMessage()
    {
        return progressContext.getProgressMessage();
    }

    public void
    setActiveProgressReportContext( ProgressReportContext _prc )
    {
        progressContext = _prc;
    }


    public String
    getUniqName()
    {
        return getName();
    }
    public String
    getDisplayName()
    {
        return display_name;
    }
    public String
    getDType()
    {
        return dtype;
    }
    public String
    getUOM()
    {
        return uom;
    }
    public String
    getUrl()
    {
        return url;
    }
    public String
    getDescription()
    {
        return description;
    }


    // --------- data reading methods ---------------------------

    public void
    createDataMatrix( int number_of_cnodes, int number_of_threads )
    {
        progressContext.setNextStep( 1 );
        progressContext.setProgressMessage( "Create empty data matric..." );
        matrix = new Matrix( metric_value,  number_of_cnodes, number_of_threads );
        progressContext.setAchievedStep();

/*
                header.print();
                index.print();
                rows.print();
                matrix.print();*/
    }


    public void
    createDataMatrix( byte[] metric_index, byte[] metric_data, int number_of_cnodes, int number_of_threads ) throws BadCubeReportLayoutException, Exception
    {
        progressContext.setNextStep( 0.03 );
        progressContext.setProgressMessage( "Read index header for Metric \"" + display_name + "\"..." );
        IndexHeader header = new IndexHeader( metric_index );
        progressContext.setAchievedStep();

        progressContext.setNextStep( 0.03 );
        progressContext.setProgressMessage( "Create index for Metric \"" + display_name + "\"..." );
        Index     index     = header.getIndex( number_of_cnodes, number_of_threads ); // creates sparse or densed index with given parameters.
        Endianess endianess = header.getEndianess();
        progressContext.setAchievedStep();

        progressContext.setNextStep( 0.24 );
        progressContext.setProgressMessage( "Load data rows for Metric \"" + display_name + "\"..." );
        DataRows rows = DataChooser.getDataRows( metric_data, metric_value, endianess );
        progressContext.setAchievedStep();

        progressContext.setNextStep( 0.70 );
        progressContext.setProgressMessage( "Create data container for Metric \"" + display_name + "\"..." );
        matrix = new Matrix( rows, index, number_of_cnodes, number_of_threads );

        // code to less memory footprint... relies on fact, that all data was loaded to memory.
        header = null;
        index  = null;
        rows.removeRowsFromMemory();
        rows = null;

        progressContext.setAchievedStep();
    }




    // -------------- value seting methods ---------------------

    public void
    set_sev( Cnode cnode, Location loc, String val ) throws NotEnumeratedCnodeException
    {
        int CID = enumeration.getCID( cnode );
//              System.out.println(CID);
        if ( matrix != null )
        {
            matrix.setValue( CID, loc.getId(), val );
        }
    }

    // ------------- value calculation methods -----------------


    public Value
    get_sev( Cnode cnode, CubeCalculationFlavour cf, Location loc, CubeCalculationFlavour lf ) throws NotEnumeratedCnodeException
    {
        int   CID   = enumeration.getCID( cnode );
        Value value = ( matrix != null ) ? matrix.getValue( CID, loc.getId() ) : this.metric_value;
        return value;
// //           return 23;
    }

    public Value
    get_sev( Cnode cnode, CubeCalculationFlavour cf ) throws NotEnumeratedCnodeException
    {
        return this.metric_value;
// //           return 23;
    }

    public Value
    get_sev( Cnode cnode, CubeCalculationFlavour cf, LocationGroup lg, CubeCalculationFlavour lgf ) throws NotEnumeratedCnodeException
    {
        Value value = metric_value.clone();
        if ( lgf == CubeCalculationFlavour.EXCLUSIVE )
        {
            return value;
        }
        for ( scalasca.cubex.cube.Vertex _location : lg.getAllChildren() )
        {
            value.addValue( get_sev( cnode, cf, ( Location )_location, CubeCalculationFlavour.INCLUSIVE ) );
        }
        return value;
    }

    public Value
    get_sev( Cnode cnode, CubeCalculationFlavour cf, SystemTreeNode stn, CubeCalculationFlavour stnf ) throws NotEnumeratedCnodeException
    {
        Value value = metric_value.clone();
        if ( stnf == CubeCalculationFlavour.EXCLUSIVE )
        {
            return value;
        }
        for ( scalasca.cubex.cube.Vertex _stn : stn.getAllChildren() )
        {
            value.addValue( get_sev( cnode, cf, ( SystemTreeNode )_stn, CubeCalculationFlavour.INCLUSIVE ) );
        }
        for ( scalasca.cubex.cube.Vertex _locationgroup : stn.getAllGroups() )
        {
            value.addValue( get_sev( cnode, cf, ( LocationGroup )_locationgroup, CubeCalculationFlavour.INCLUSIVE ) );
        }
        return value;
    }



    public void
    createEnumeration( ArrayList<scalasca.cubex.cube.Cnode> root_cnodes )
    {
        enumeration = new CnodesEnumeration( root_cnodes );
    }
}
